#include <Adafruit_NeoPixel.h>
#define LED_PIN 2
Adafruit_NeoPixel pixels(1, LED_PIN, NEO_GRB + NEO_KHZ800);

// Please tune the following value if the clock gains or loses.
// Theoretically, standard of this value is 60000.
#define MILLIS_PER_MIN 60071 // milliseconds per a minute

#define STEPS_PER_ROTATION 4096 // steps for a full turn of minute rotor
#define POS 14
#define MOTOR_NUM 3
#define MOTOR_PIN 4

// minimum wait for a single step of stepper
#define DELAY 1

// ports used to control the stepper motor
// if your motor rotate to the opposite direction, 
// change the order as {4, 5, 6, 7};
int port[MOTOR_NUM][MOTOR_PIN] = {
  {7, 6, 5, 4},
  {10, 8, 1, 0},
  {21, 19, 18, 9}
};
int initPos[MOTOR_NUM] = {50, 70, 70};

long stepToGo[MOTOR_NUM] = {0};

// sequence of stepper motor control
int seq[8][MOTOR_PIN] = {
  {  LOW, HIGH, HIGH,  LOW},
  {  LOW,  LOW, HIGH,  LOW},
  {  LOW,  LOW, HIGH, HIGH},
  {  LOW,  LOW,  LOW, HIGH},
  { HIGH,  LOW,  LOW, HIGH},
  { HIGH,  LOW,  LOW,  LOW},
  { HIGH, HIGH,  LOW,  LOW},
  {  LOW, HIGH,  LOW,  LOW}
};

void rotateSet(int num, long step) {
  stepToGo[num] += step;
}

void rotateGo(void) {
  static long phase[MOTOR_NUM] = {0};
  int dt = 20;
  int done = 0;
  int delta;

  while(!done) {
    done = 1;
    for(int m = 0; m < MOTOR_NUM; m++) {
      long step = stepToGo[m];
      if(step == 0) {
        for(int i = 0; i < MOTOR_PIN; i++) {
          digitalWrite(port[m][i], LOW);
        }
        continue;
      }
      else if(step > 0) {
        stepToGo[m]--;
        delta = 1;
      }
      else {
        stepToGo[m]++;
        delta = 7;
      }
      done = 0;
      phase[m] = (phase[m] + delta) % 8;
      for(int i = 0; i < MOTOR_PIN; i++) {
        digitalWrite(port[m][i], seq[phase[m]][i]);
      }
    }
    delay(dt);
    if(dt > DELAY) {
      dt--; // accelerate
    }
  }
  // power cut
  for(int m = 0; m < MOTOR_NUM; m++) {
    for(int i = 0; i < MOTOR_PIN; i++) {
      digitalWrite(port[m][i], LOW);
    }
  }
}

void setup() {
  for(int m = 0; m < MOTOR_NUM; m++) {
    for(int i = 0; i < MOTOR_PIN; i++) {
      pinMode(port[m][i], OUTPUT);
    }
  }
  pixels.setPixelColor(0, pixels.Color(255, 0, 0));    //Colorメソッド内の引数の順番は赤、緑、青
  pixels.show();
  for(int m = 0; m < MOTOR_NUM; m++) {
    rotateSet(m, -STEPS_PER_ROTATION); // reverse rotation to endstop
  }
  rotateGo();
  for(int m = 0; m < MOTOR_NUM; m++) {
    rotateSet(m, STEPS_PER_ROTATION / 2 + initPos[m]); // show 0
  }
  rotateGo();
  pixels.setPixelColor(0, pixels.Color(0, 255, 0));    //Colorメソッド内の引数の順番は赤、緑、青
  pixels.show();
}

int order[MOTOR_NUM][POS] = {
  {0, 10, 5, 3, 8, 6, 2, 13, 9, 1, 11, 4, 12, 7},
  {0, 13, 7, 10, 8, 12, 6, 3, 11, 5, 1, 4, 2, 9},
  {0, 2, 7, 10, 4, 12, 5, 8, 3, 11, 1, 6, 9, 13}
};

int currentPos[MOTOR_NUM] = {0};

void setNum(int m, int n) {
  int i, pos, newPos, advance;

  for(i = 0; i < POS; i++) {
    if(order[m][i] == n)
      newPos = i;
  }
  pos = (newPos * STEPS_PER_ROTATION) / POS;
  advance = (pos - currentPos[m] + STEPS_PER_ROTATION) % STEPS_PER_ROTATION; // always CW rotation
  rotateSet(m, advance);
  currentPos[m] = pos;
}

void loop() {
  long min;
  static long prev_min;
  static int n;

  delay(500);
  n = (n + 1) % POS;
  setNum(0, n);
  setNum(1, n);
  setNum(2, n);
  rotateGo();

#if 0
min = millis() / MILLIS_PER_MIN;
  if(digitalRead(16) == LOW || min != prev_min) {
    advanceStep(1);
  }
  prev_min = min;
#endif
}
